
/* Includes files and external references ...................................*/

#include <stdtyp.l>
#include <moteur.h>                    /* vmk.c and vmio.c interface         */
#include <drv.h>                       /* drv.c interface                    */

#include <rdrv.h>                      /* drv.c tags interface               */
#include <memoire.h>                   /* mem_buf.c interface                */

#include <vmkm.txt>                    /* TSK_INIT_DATA TASK_INIT_BSS        */

#include "./myio.h"                    /* Prototypes for "myio_xxx" procs    */
#include "./myapp.h"                   /* Constant and structs for myapp.c   */

/* Internal global variables for the user task ------------------------------*/

int                 mystack[512]     ; // The stack of our task 
int                 myapp_taskid     ; // Our task id
int                 myapp_memuid     ; // Our memory user id


int                 led_iod          ; // IOD of device LED\DEV0
int                 led_iod0         ; // IOD of subchannel 0 of LED\DEV0
int                 led_iod1         ; // IOD of subchannel 1 of LED\DEV0
int                 led_iod2         ; // IOD of subchannel 2 of LED\DEV0

static int     hsled            (int ,int                                   ) ;


/*****************************************************************************/
int init_vmk_fsm(Task_grp *g,char *mod, char*conf)
  {

    return 0                         ; // Exit without any error
  }


/*****************************************************************************/
int init_myapp(Task_grp *g,char *mod, char*conf)
  {

    int ret  ;

    create_task("app"           ,      // Taskgroup name
                mytask_proc     ,      // Task entry point
                myfree_proc     ,      // Task termination call-back
                mystack         ,      // Stack address
                sizeof(mystack) ,      // Stack size in bytes
                PRIO_TSK_MIN    ,      // Task Priority
                HNULL           ,      // Parameter for "mytask_proc"
                TASK_INIT_BSS  +       // Init Flags
                TASK_INIT_DATA +       //
                TASK_INIT_CODE  ,      //
                &myapp_taskid     )  ; // Task_id = 0x12FFFF00 + LW


    start_task(myapp_taskid,           // Send an event to VMK in order
               &ret          )       ; // to request the task start


    init_telecom()                   ; // Initialize driver/protocol/services
                                       // API
    return 0                         ; // Exit without any error
  }


/*****************************************************************************/
static INT mytask_proc(void *param)
  {

    int res = 0                      ; 

    open_led()                       ; // Open the LED driver

    hsled(0,1)                       ; // switch on the red led
    hsled(1,1)                       ; // switch on the green led
    hsled(2,1)                       ; // switch on the yellow led
   
                                       // after wait for nothing
    waitevt_task(HNULL    ,            // Address of waiting list
                 0        ,            // Size of "waitlist[]"
                 30000    ,            // Wait 3O s
                 0        ,            // Do not purge previous events
                 &res       )        ; // Return code

    close_led()                      ; // Close the LED driver

    return 0                         ;
  }


/*****************************************************************************/
static int myfree_proc(void)
  {
    int             ret              ; // Procedurs return code

    free_memuid(myapp_memuid,          // Frees a user id for alloc_buf
                0x00000007  ,          // mask = buffers/links/huge
                &ret          )      ; //
    return 0                         ;
  }


/*****************************************************************************/
static void open_led(void)
  {
   
    int ret = 0 ;

    ret = myio_open(DRVLED,DEV0,"",&led_iod)  ; // Open LED\DEV0

    ret = myio_alloc_sub(led_iod,"LEDNAME=RED"    ,&led_iod0); // Allocates RED
    ret = myio_alloc_sub(led_iod,"LEDNAME=GREEN"  ,&led_iod1); // Allocates GREEN
    ret = myio_alloc_sub(led_iod,"LEDNAME=YELLOW" ,&led_iod2); // Allocates YELLOW

    if (ret) ret ++ ;
  }



/*****************************************************************************/
static void close_led(void)
  {

    myio_free_sub(led_iod0)          ; // Frees RED
    myio_free_sub(led_iod1)          ; // Frees GREEN
    myio_free_sub(led_iod2)          ; // Frees YELLOW

    myio_close(led_iod)              ; // Closes LED device and driver
  }

/*****************************************************************************/
static int  hsled(int n , int state)

  {

    int ret = 0 ;

    switch ( n )
      {
        case 0 :
          ret = myio_setval(led_iod0, "CMD=POPALL" );
          ret = myio_setval(led_iod0, "PATTERN=ON:20,OFF:10\nCMD=PUSH")  ;
        break;

        case 1 :
          ret = myio_setval(led_iod1, "CMD=POPALL" );
          ret = myio_setval(led_iod1, "PATTERN=ON:5,OFF:5\nCMD=PUSH")  ;
        break;

        case 2 :
          ret = myio_setval(led_iod2, "CMD=POPALL" );
          ret = myio_setval(led_iod2, "PATTERN=ON:10,OFF:10\nCMD=PUSH")  ;

        break;
      }
    
    return ret ;

  }

